;;; mw-auth.el --- Logging into MediaWiki            -*- lexical-binding: t; -*-

;; Copyright (C) 2014 Free Software Foundation, Inc.

;; Author: Ivan Shmakov <ivan@siamics.net>
;; Version: 0.1
;; Keywords: web-services, comm

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:

;; This module currently provides just one command (mw-auth-log-in),
;; which authenticates the user against a MediaWiki instance.  It
;; relies on auth-source.

;;; Code:
(require 'auth-source)
(require 'mw-api)
(require 'mw-buffer)
(require 'mw-info)			; for mw-info-describe-*

;;;###autoload
(defun mw-auth-log-in (&optional target alist)
  "Log into the MediaWiki instance associated with the current buffer.
TARGET is the buffer to print the authentication result to.  If nil,
this function will use *MediaWiki*, erasing it beforehand.

ALIST are the extra parameters to pass to the MediaWiki API call."
  (interactive
   (progn
     (unless mw-buffer-endpoint-uri
       (call-interactively 'mw-buffer-associate-endpoint))
     nil))
  (let* ((uri-string mw-buffer-endpoint-uri)
	 (uri (url-generic-parse-url uri-string))
	 (auth-source-creation-prompts
	  '((password . "Enter MediaWiki password for %h:%p: ")))
	 (au  (auth-source-search
	       :host (url-host uri)
	       :port (url-port uri)
	       ;; FIXME: also consider, say, url-filename, url-user?
	       :require '(:user :secret)
	       :create  t
	       :max 1))
	 (q-1 (mw-api-adjoin
	       "lgname" (plist-get (car au) :user)
	       (mw-api-adjoin "lgpassword"
			      (let ((s (plist-get (car au) :secret)))
				(if (stringp s)
				    s
				  (funcall s)))
			      alist)))
	 (result-1
	  (mw-info-describe-1 target
			      (lambda (result)
				(cdr (assoc 'login result)))
			      "login" q-1))
	 (login-1   (assoc 'login result-1))
	 (outcome-1 (cdr (assoc 'result login-1))))
    ;; NB: necessary for the mw-info-describe-1 call below
    (setq mw-buffer-endpoint-uri uri-string)
    (cond ((equal "Success" outcome-1)
	   (message "Successfully logged into (older) MediaWiki at %s"
		    uri-string)
	   ;; .
	   result-1)
	  ((equal "NeedToken" outcome-1)
	   (let ((result-2
		  (mw-info-describe-1 (current-buffer) ; inhibit erase-buffer
				      (lambda (result)
					(insert "\n")
					(cdr (assoc 'login result)))
				      "login"
				      (mw-api-adjoin
				       "lgtoken" (cdr (assoc 'token login-1))
				       q-1))))
	     (if (equal "Success"
			(cdr (assoc 'result
				    (cdr (assoc 'login result-2)))))
		 (message "Successfully logged into MediaWiki at %s"
			  uri-string)
	       (message "Failed to log into MediaWiki at %s"
			uri-string)
	       (pop-to-buffer (current-buffer)))
	     ;; .
	     result-2))
	  (t
	   (message "Failed to log into MediaWiki at %s"
		    uri-string)
	   (pop-to-buffer (current-buffer))
	   ;; .
	   result-1))))

(provide 'mw-auth)
;;; mw-auth.el ends here
