;;; mw-info.el --- Displaying information retrieved from MediaWiki  -*- lexical-binding: t; -*-

;; Copyright (C) 2014 Free Software Foundation, Inc.

;; Author: Ivan Shmakov <ivan@siamics.net>
;; Version: 0.1
;; Keywords: web-services, comm

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:

;; The goal for this module is to provide both user-accessible
;; commands for retrieving metadata (but not actual page content) from
;; MediaWiki, and a common (extensible) infrastructure for rendering
;; MediaWiki API requests and responses.

;;; Code:
(require 'mw-api)
(require 'mw-buffer)

(defvar mw-info-buffer "*MediaWiki*")

(defun mw-info-describe-1 (target handler action parameters)
  (let ((res (mw-api-call mw-buffer-endpoint-uri action parameters))
	info)
    (set-buffer (or target (get-buffer-create mw-info-buffer)))
    (unless target
      (erase-buffer))
    (setq info  (funcall handler res))	; also for side-effects
    (insert "* Result\n")
    (dolist (ent info)
      (insert (format "%s: %s\n" (car ent) (cdr ent))))
    (insert "\n* " (capitalize action) "\n")
    (dolist (ent parameters)
      (insert (format "%s: %s\n" (car ent) (cdr ent))))
    ;; .
    res))

;;;###autoload
(defun mw-info-describe-page (&optional target alist)
  "Describe the revision or page associated with the current buffer.
TARGET is the buffer to print the information to.  If nil, this
function will use *MediaWiki*, erasing it beforehand.

ALIST are the extra parameters to pass to the MediaWiki API call."
  (interactive
   (progn
     (unless mw-buffer-endpoint-uri
       (call-interactively 'mw-buffer-associate-endpoint))
     (unless (or mw-buffer-page-id
		 mw-buffer-page-revision
		 mw-buffer-page-title)
       (call-interactively 'mw-buffer-associate-page))
     nil))
  ;; .
  (prog1
      (mw-info-describe-1 target
			  (lambda (res)
			    (let ((page (mw-api-get '(query pages t)
						    res)))
			      (when (assoc 'missing page)
				(insert "*** No such page ***\n\n"))
			      ;; .
			      page))
			  "query"
			  (mw-buffer-page-query
			   (mw-api-adjoin "prop" "info" alist)))
    (pop-to-buffer (current-buffer))))

;;;###autoload
(defun mw-info-describe-user (&optional target alist)
  "Describe the user currently logged into MediaWiki.
TARGET is the buffer to print the information to.  If nil, this
function will use *MediaWiki*, erasing it beforehand.

ALIST are the extra parameters to pass to the MediaWiki API call."
  (interactive
   (progn
     (unless mw-buffer-endpoint-uri
       (call-interactively 'mw-buffer-associate-endpoint))
     nil))
  ;; .
  (prog1
      (mw-info-describe-1 target
			  (lambda (result)
			    (cdr (assoc 'userinfo
					(cdr (assoc 'query result)))))
			  "query"
			  (mw-api-adjoin "meta" "userinfo" alist))
    (pop-to-buffer (current-buffer))))

(provide 'mw-info)
;;; mw-info.el ends here
